#ifndef COMMON_H
#define COMMON_H




//------------------------------------------------------//
//  C++
//------------------------------------------------------//
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <map>
#include <utility>
#include <math.h>
#include <sys/stat.h>
#include <cmath>
#include <complex>
#include <cstdarg>
#include <typeinfo>



//------------------------------------------------------//
//  QT
//------------------------------------------------------//
#include <QtGui>
#include <QDir>
#include <QtGui>
#include <QString>
#include <QDebug>
#include <QDir>
#include <QObject>

#include <QProgressBar>

//------------------------------------------------------//
//  boost
//------------------------------------------------------//
#ifndef Q_MOC_RUN
#include <boost/shared_ptr.hpp>
#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/multi_array.hpp>
#include <boost/array.hpp>
#include <boost/lexical_cast.hpp>
#endif


//------------------------------------------------------//
//  libharkio
//------------------------------------------------------//
#include "libharkio/libharkio.h"
#include "libharkio/textdata.h"


#include "fft.hpp"


//------------------------------------------------------//
//  util
//------------------------------------------------------//
#include "utils.hpp"






//------------------------------------------------------//
// TARGET combboxの値
//------------------------------------------------------//
enum TARGET
{
    DISP_TF   =  0,
    DISP_IFFT =  1,
    DISP_DIFF =  2
};


//------------------------------------------------------//
//  x,y軸　combo box
//------------------------------------------------------//
enum X_Y_COMBO
{
    DISP_TIME =  0 ,
    DISP_FREQ =  0 ,
    DISP_SRC  =  1 ,
    DISP_MIC  =  2
};


//------------------------------------------------------//
//  z軸　combo box
//------------------------------------------------------//
enum z_COMBO
{
    DISP_AMP   =  0,
    DISP_db    =  1,
    DISP_PHASE =  2,
    DISP_REAL  =  3
};


enum OTHER
{
    ERROR           = -1,
    DISP_UNDEFINE   = 0
};



//------------------------------------------------------//
//プロットスタイル
//------------------------------------------------------//
enum PLOT_STYLE
{
    DISP_DOTS        =  1 ,
    DISP_POINTS      =  2 ,
    DISP_LINES       =  3 ,
    DISP_LINESPOINTS =  4
};


//------------------------------------------------------//
//カラーマップ
//------------------------------------------------------//
enum COLOR_MAP
{
    DISP_PM3D       =    1,
    DISP_PM3D_MAP   =    2,
    DISP_PM3D_AT_B  =    3
};




//------------------------------------------------------//
//表面
//------------------------------------------------------//
enum surface
{
    DISP_LINE   =  1,
    DISP_MESH   =  2
};




//------------------------------------------------------//
//軸の統一用
//------------------------------------------------------//
enum AXIS_X
{
    AXIS_NONE  =  0,
    AXIS_TIME  =  1,
    AXIS_FREQ  =  2,
    AXIS_SRC   =  3,
    AXIS_MIC   =  4
};


//------------------------------------------------------//
// y軸
//------------------------------------------------------//
enum AXIS_Y
{
    AXIS_AMP   =  1,
    AXIS_db    =  2,
    AXIS_PHASE =  3
};



//------------------------------------------------------//
//
//------------------------------------------------------//
enum  UNDEFINE
{
    UNDEFINE   =  0
};



//------------------------------------------------------//
// 軸を表す表示マクロ
//------------------------------------------------------//
enum AXIS_XYZ
{
    AXIS_X  =   INT_MAX-2,
    AXIS_Y  =   INT_MAX-1,
    AXIS_Z  =   INT_MAX-0
};


//------------------------------------------------------//
// gnuplotの表示マクロ
//------------------------------------------------------//
enum PLOTSTYLE
{
    PlotStyle_UNDEFINE     = 0,
    PlotStyle_DOTS         = 1,
    PlotStyle_POINTS       = 2,
    PlotStyle_LINES        = 3,
    PlotStyle_LINESPOINTS  = 4
};


enum COLORMAP
{
    ColorMap_UNDEFINE     =  0,
    ColorMap_PM3D         =  1,
    ColorMap_PM3D_MAP     =  2,
    ColorMap_PM3D_AT_B    =  3
};


enum SURFACE
{
    Surface_UNDEFINE      =  0,
    Surface_LINE          =  1,
    Surface_MESH          =  2
};



//------------------------------------------------------//
// 現在、実行中ステータス
//------------------------------------------------------//
enum RUN_STAT
{
    START   =  10,
    STOP    =  20,
    RUNNING =  30
};


//------------------------------------------------------//
// 現在、実行中処理
//------------------------------------------------------//
enum RUN_KIND
{
    RUN_IMP   = 1,
    RUN_TSP   = 2,
    RUN_MIC   = 3,
    RUN_NOISE = 4,
    RUN_MUSIC = 5,
    RUN_GTF   = 6,
    RUN_MUSIC_CUI = 7,
    RUN_GTF_CUI   = 8
};



//-------------------------------------
//  stackedWidget ページ番号
//-------------------------------------
enum IN_PAGE
{
    DISP_scrProgressBar   = 0 ,
    DISP_scrIMPtemplate   = 1 ,
    DISP_scrTSPtemplate   = 2 ,
    DISP_scrMICtemplate   = 3 ,
    DISP_scrNOISEtemplate = 4 ,
    DISP_scrM2PG          = 5 ,
    DISP_scrTransfer      = 6 ,
    DISP_scrGTF           = 7 ,
    DISP_scrWork          = 8
};



//-------------------------------------
//  stackedWidget_2 ページ番号
//-------------------------------------
enum OUT_PAGE
{
    DISP_scrBlackPage   = 0,
    DISP_srcIMPout      = 1,
    DISP_srcTSPout      = 2,
    DISP_srcMICout      = 3,
    DISP_srcNOISEout    = 4,
    DISP_srcM2PGout     = 7,
    DISP_scrGTFout      = 8

};



//-------------------------------------
//  M2PG table ページ番号
//-------------------------------------
enum OUT_M2PG_TABLE
{
    DISP_M2PG_FMT_VERSION        = 1,
    DISP_M2PG_FORMATID           = 2,
    DISP_M2PG_NUM_OF_MIC         = 3,
    DISP_M2PG_FFT_LENGTH         = 4,
    DISP_M2PG_IR_LENGTH          = 5,
    DISP_M2PG_SAMPLING           = 6,
    DISP_M2PG_MIC_POSITION       = 7,
    DISP_M2PG_VERSION            = 10,
    DISP_M2PG_NUM_OF_AZIMUTH     = 11,
    DISP_M2PG_NUM_OF_DISTANCE    = 12,
    DISP_M2PG_SOURCE_LOCATION    = 13,
    DISP_M2PG_TRANSFER_FUNCTION  = 14,
    DISP_M2PG_DATA               = 17,
    DISP_M2PG_PLACE              = 18,
    DISP_M2PG_ROBOT              = 19,
    DISP_M2PG_ARRAY_TYPE         = 20,
    DISP_M2PG_CREATOR            = 21,
    DISP_M2PG_NOTES              = 22
};




//-------------------------------------
//  gtf table ページ番号
//-------------------------------------
enum OUT_GTF_TABLE
{
    DISP_FMT_VERSION        = 1,
    DISP_FORMATID           = 2,
    DISP_NUM_OF_MIC         = 3,
    DISP_FFT_LENGTH         = 4,
    DISP_IR_LENGTH          = 5,
    DISP_SAMPLING           = 6,
    DISP_MIC_POSITION       = 7,
    DISP_GTF_VERSION        = 10,
    DISP_NUM_OF_SOURCE      = 11,
    DISP_SOURCE_LOCATION    = 12,
    DISP_TRANSFER_FUNCTION  = 13,
    DISP_DATA               = 16,
    DISP_PLACE              = 17,
    DISP_ROBOT              = 18,
    DISP_ARRAY_TYPE         = 19,
    DISP_CREATOR            = 20,
    DISP_NOTES              = 21
};






const std::string TSP_TO_IMPULSE = "calcSysImp";	   // TSPからインパルス応答に変換するプログラム
const std::string HARK_TOOLS_DIR = "harktool4_utils";  // 標準の外部プログラムのサブディレクトリ名
const std::string IMPULSE_ORG    = "16384.little_endian.tsp";
const std::string TEMPORARY_PATH = ".harktool";



typedef double HT_DATA_TYPE;
typedef std::complex<HT_DATA_TYPE> HT_COMPLEX_DATA_TYPE;



//------------------------------------------------------//
// ファイルの種類を判断する
//------------------------------------------------------//
enum  class HarkConfigFileType
{
    Unknown,
    ImpulseResponseListFile,
    TSPListFile,
    MicArrayLocationFile,
    NoiseLocationFile
};

enum class HarkConfigFileFormatType
{
    Unknown,
    Text,
    XML
};


// ファイルの種類を判断する（HARK-configファイルの判別用）
HarkConfigFileType GetHarkConfigTextFileType(const std::string& filename);
HarkConfigFileType GetHarkConfigXMLFileType(const std::string& filename);
std::pair<HarkConfigFileFormatType, HarkConfigFileType> GetHarkConfigFileType(const std::string& filename);


QString getUserName() ;







#endif // COMMON_H
